sbit LCD_RS at PORTB2_bit;
sbit LCD_EN at PORTB3_bit;
sbit LCD_D4 at PORTB4_bit;
sbit LCD_D5 at PORTB5_bit;
sbit LCD_D6 at PORTB6_bit;
sbit LCD_D7 at PORTB7_bit;

sbit LCD_RS_Direction at DDB2_bit;
sbit LCD_EN_Direction at DDB3_bit;
sbit LCD_D4_Direction at DDB4_bit;
sbit LCD_D5_Direction at DDB5_bit;
sbit LCD_D6_Direction at DDB6_bit;
sbit LCD_D7_Direction at DDB7_bit;


void setup_MCU();
void display_common();
float read_voltage(unsigned short channel);
void convert_for_segments(unsigned long raw_data);
float read_humidity();
float read_temperature();
void display_data(unsigned char x, unsigned char y, float value);


void main()
{
     setup_MCU();
     display_common();
     while(1)
     {
         display_data(1,2,read_temperature());
         display_data(1,4,read_humidity());
     }
}


void setup_MCU()
{
     DDRC = 0x00;
     DDRD = 0xFF;
     PORTC = 0x00;
     PORTD = 0x00;
     TCCR0 = 0x00;
     TCNT0 = 0x00;
     TCCR1A = 0x00;
     TCCR1B = 0x00;
     TCNT1H = 0x00;
     TCNT1L = 0x00;
     ICR1H = 0x00;
     ICR1L = 0x00;
     OCR1AH = 0x00;
     OCR1AL = 0x00;
     OCR1BH = 0x00;
     OCR1BL = 0x00;
     ASSR = 0x00;
     TCCR2 = 0x00;
     TCNT2 = 0x00;
     OCR2 = 0x00;
     MCUCR = 0x00;
     TIMSK = 0x00;
     SFIOR = 0x00;
     SREG_I_bit = 0;
     LCD_Init();
     ADC_Init();
     Lcd_Cmd(_LCD_CLEAR);
     Lcd_Cmd(_LCD_CURSOR_OFF);
     delay_ms(500);
}


void display_common()
{
     unsigned char disp1[] = "Temperature:";
     unsigned char disp2[] = "Relative Humidity:";
     unsigned char disp3[] = "C";
     unsigned char disp4[] = "%";
     Lcd_Out(1,1,disp1);
     Lcd_Out(2,7,176);
     Lcd_Out(2,7,disp3);
     Lcd_Out(3,1,disp2);
     Lcd_Out(4,6,disp4);
}


float read_voltage(unsigned short channel)
{
      register unsigned long raw_v = 0;
      rx float vin = 0;
      unsigned short samples = 0;
      
      for(samples = 0; samples < 64; samples++)
      {
                raw_v += adc_read(channel);
      }
      
      raw_v >>= 6;
      vin = (raw_v * 0.0048875);
      return vin;
}


float read_humidity()
{
    register float HSM_20G = 0;
    register float v = 0;
    v = read_voltage(0);
    HSM_20G = ((3.71 * v * v * v) - (20.65 * v * v) + (64.81 * v) - 27.44);
    return HSM_20G;
}


float read_temperature()
{
    register float HSM_20G = 0;
    register float v = 0;
    v = read_voltage(1);
    HSM_20G = ((5.26 * v * v * v) - (27.34 * v * v) + (68.87 * v) - 17.81);
    return HSM_20G;
}


void display_data(unsigned char x, unsigned char y, float value)
  {
    unsigned char c = 0;
    register unsigned long conv = 0;
    conv = (value * 100.0);
    c = (conv * 0.001);
    Lcd_Chr(y, x, 48 + c);
    c = ((conv / 100) % 10);
    Lcd_Chr_CP(48 + c);
    Lcd_Chr_CP('.');
    c = ((conv / 10) % 10);
    Lcd_Chr_CP(48 + c);
    c = (conv % 10);
    Lcd_Chr_CP(48 + c);
    delay_ms(100);
  }